#' getMOM:
#' Calculate  multiple of means(MOMs) of angiogenic and anti-angiogenic factors (sVEGFR1, sEng, and PLGF) and  mean arterial pressure (MAP) according to the PRB/NICHD population.
#' @param data: A data frame with the following columns:
#'
#'     SampleGA: Gestational Age (weeks)
#'
#'     Weight: Maternal weight (kg)
#'
#'     Height: Maternal height (cm)
#'
#'     Age: Maternal age (years)
#'
#'     Nulliparity: 1 for nulliparous mothers and 0 for multiparous
#'
#'     Smoking: 1 for smokers and 0 for non smokers
#'
#'     CHTN: 1 for chronic hypertensive mother and 0 otherwise
#'
#'     PEHistory: 1 if there is a history of preeclampsia and 0 otherwise
#'
#'     sVEGFR1: soluble vascular endothelial growth factor receptor 1 (pg/mL)

#'     sEng: soluble endoglin (ng/mL)
#'
#'     PLGF: Placental growth factor  (pg/mL)
#'
#'     MAP: Mean arterial pressure (mmHg)
#'
#'
#'
#' @return The input data frame with  additional columns for the MOMs.
#' @description   This function calculates the multiple of means(MOMs) of angiogenic and anti-angiogenic factors (sVEGFR1, sEng, and PLGF) and  mean arterial pressure (MAP).
#' @examples
#'
#' @references
#'
#' Contact: Adi Laurentiu Tarca <atarca@med.wayne.edu>
#'
#' @export


getMOM<-function(data)

{
  makeX = function(cof, ano) {
    for (n in rownames(cof)) {
      if (n == "(Intercept)") {
        X = rep(1, dim(ano)[1])
      }
      if (n %in% names(ano)) {
        X = cbind(X, ano[, n])
      }
      if (grepl(":", n)) {
        v1 = unlist(strsplit(n, split = ":"))[1]
        v2 = unlist(strsplit(n, split = ":"))[2]
        X = cbind(X, ano[, v1] * ano[, v2])
      }
    }
    X
  }


  data$mid = paste0("m", seq(1, nrow(data)))
  rownames(data) = data$mid
  origdata = data


  if(any(!c("Weight","Height","Age","PEHistory","Smoking","CHTN","SampleGA","Nulliparity") %in% colnames(data)))
  {
    stop("Missing columns")

  }

  if(all(!c("sVEGFR1","sEng","PLGF","MAP") %in% colnames(data)))
  {
    stop("Missing columns")

  }

  ang = c("sVEGFR1","sEng","PLGF","MAP")
  ang= ang[ang %in% colnames(data)]
  ano=data.frame(data[,c("Weight","Height","Age","PEHistory","Smoking","CHTN","SampleGA","Nulliparity",ang)],stringsAsFactors = F)
  ano[]=lapply(ano, function(x){as.numeric(as.character(x))})

  ano<-ano[ano$SampleGA>=8&ano$SampleGA<37,]



  ano$mwkg<-ano$Weight-75
  ano$HTKG<-ano$Height-163
  ano$Age<-ano$Age-24
  ano$Interval <- cut(ano$SampleGA,c(8,16,20,24,28,32,37),right=F)
  ano$Smok<- ano$Smoking
  ano$Nuli<- ano$Nulliparity
  ano$HistPe<- ano$PEHistory

  levels(ano$Interval)=paste0("Interval",c(1:6))
  ano$Interval<-as.character(ano$Interval)
  ano$intvstart<- ifelse(ano$SampleGA<16,8,
                         ifelse(ano$SampleGA<20,16,
                                ifelse(ano$SampleGA<24,20,
                                       ifelse(ano$SampleGA<28,24,
                                              ifelse(ano$SampleGA<32,28,
                                                     ifelse(ano$SampleGA<37,32,NA))))))
  ano$SampleGA<-ano$SampleGA-ano$intvstart

  momcoefmat<-momsrcdata#(read.csv("mommod_coefmatrix.csv",row.names = 1,check.names = F))
  angmeans<-NULL
  for( i in 1:length(ang))
  {
    cof= as.data.frame(t(momcoefmat[paste(ang[i],ano[,"Interval"],sep="_"),]))
    cof[]=lapply(cof, function(x){as.numeric(as.character(x))})

    cof=as.matrix(cof)
    rownames(cof)= colnames(momcoefmat)

    pmat = makeX(cof, ano = ano)
    qua = diag(pmat %*% cof)
    angmeans<-cbind(angmeans,qua)

  }
  colnames(angmeans)<- paste0(ang,"Mean")
  rownames(angmeans)<-rownames(ano)
 angmeans<-data.frame(angmeans,stringsAsFactors = F)
  ano<-cbind(ano,angmeans)
  angp<-ang

  for ( i in 1:length(angp))
  {
    ano[,paste0(angp[i],"MOM")] <- log10(ano[,angp[i]]) -ano[,paste0(angp[i],"Mean")]
    origdata[rownames(ano),paste0(angp[i],"MOM")] <- (log10(ano[,angp[i]]) -ano[,paste0(angp[i],"Mean")])

  }
 rownames(origdata)<- NULL
 origdata$mid<-NULL

  return(origdata)
}


