#' riskALLPE:
#' Calculates the posterior risk of preeclampsia based on the prior risk and the centiles of PLGF, sVEGFR1, sENG and MAP.
#' @param data: A data frame with the following columns:
#'
#'    SampleGA: Gestational Age (weeks)
#'
#'    sVEGFR1MOM: Log10 transformed MOM of soluble vascular endothelial growth factor receptor 1 (pg/mL)
#'
#'    sEngMOM: Log10 transformed MOM of soluble endoglin (ng/mL)
#'
#'    PLGFMOM: Log10 transformed MOM of Placental growth factor  (pg/mL)
#'
#'    MAPMOM: Log10 transformed MOM of Mean arterial pressure (mmHg)
#'
#'    LogPriorRisk: Log transformed prior risk of preeclampsia
#'
#'
#' @return The input data frame with  an additional column for the posterior risk of preeclampsia.
#' @description   This function calculates the posterior risk of preeclampsia.
#' @examples
#'
#' @references
#'
#' Contact: Adi Laurentiu Tarca <atarca@med.wayne.edu>
#'
#' @export


riskALLPE<-function(data)
{

data$mid = paste0("m", seq(1, nrow(data)))
rownames(data) = data$mid
origdata = data

if(all(!c("sVEGFR1MOM","sEngMOM","PLGFMOM","MAPMOM","LogPriorRisk") %in% colnames(data)))
{
  stop("Missing columns")

}

ano=data.frame(data[,c("SampleGA","sVEGFR1MOM","sEngMOM","PLGFMOM","MAPMOM","LogPriorRisk")],stringsAsFactors = F)
ano[]=lapply(ano, function(x){as.numeric(as.character(x))})
ano<-ano[ano$SampleGA>=8&ano$SampleGA<37,]

ano$Interval <- cut(ano$SampleGA,c(8,16,20,24,28,32,37),right=F)
levels(ano$Interval)=paste0("Interval",c(1:6))
ano$Interval<-as.character(ano$Interval)

makeX = function(cof, ano) {
  for (n in rownames(cof)) {
    if (n == "(Intercept)") {
      X = rep(1, dim(ano)[1])
    }
    if (n %in% names(ano)) {
      X = cbind(X, ano[, n])
    }
    if (grepl(":", n)) {
      v1 = unlist(strsplit(n, split = ":"))[1]
      v2 = unlist(strsplit(n, split = ":"))[2]
      X = cbind(X, ano[, v1] * ano[, v2])
    }
  }
  X
}


#risksrcdata<-(read.csv("D:\\PERisk\\riskmod_coefmatrix.csv",row.names = 1,check.names = F))
#momsrcdata<- (read.csv("D:\\PERisk\\mommod_coefmatrix.csv",row.names = 1,check.names = F))
#use_data(risksrcdata,momsrcdata,internal = T,overwrite = T)
riskcoefmat<-risksrcdata
cof= as.data.frame((riskcoefmat[,ano[,"Interval"]]))
cof[]=lapply(cof, function(x){as.numeric(as.character(x))})
cof=as.matrix(cof)
rownames(cof)= rownames(riskcoefmat)
pmat = makeX(cof, ano = ano)
qua = exp(as.numeric(as.character(diag(pmat %*% cof))))




ano<-cbind(ano,PERisk=qua)
pvalues=apply(ano,1,
 function(x)
{
risk= as.numeric(as.character(x[["PERisk"]]));
emp= as.numeric(as.character(ALLEmpirical[[x[["Interval"]]]]));
pvalue= sum(emp>=risk)/length(emp)
return(pvalue)
})
ano<-cbind(ano,pvalue=pvalues)

origdata[rownames(ano),"PERisk"]=ano$PERisk
origdata[rownames(ano),"pvalue"]=ano$pvalue


rownames(origdata)<- NULL
origdata$mid<-NULL
return(origdata)
}
